

# 

showBump = function(bumps, index=0,idx = NULL, margin = 10,trend = TRUE, 
                    samples = FALSE, rawdat = NULL, categ = NULL, origscale=FALSE,...) {
  if (index == 0  && is.null(idx)){
    error('Please specify either index or (start,end) of bump')
  }
  
  if (index==0){
    bs = as.numeric(idx)[1]
    be = as.numeric(idx)[2]
  } else {
    bs = bumps$table[index,"indexStart"]
    be = bumps$table[index,"indexEnd"]    
  }
  
  # plots start and end based on margins
  ps = max(1,bs-margin)
  pe = min(be+margin,length(bumps$coef))
  
  cat(ps,pe)
  
  if (origscale){
    par(mfcol = c(2,1))
  }
  par(bty = 'n')
  plot(ps:pe,bumps$coef[ps:pe],...)
  abline(h=0,col=1)
  if (trend) {
    lines(ps:pe,bumps$fitted[ps:pe],col=1,lw=2)
    lines(bs:be,bumps$fitted[bs:be],col=3,lw=3)
  }
  if (samples) {
    if (! (is.null(rawdat) | is.null(categ))) {
      if (!origscale) {
      # remove global mean
        globmean = (rowMeans(rawdat[ps:pe, categ==0]) + rowMeans(rawdat[ps:pe, categ==1]))/2
        for (i in 1:ncol(rawdat)){
          lines(ps:pe, rawdat[ps:pe, i]-globmean,col = c(4,2)[categ[i]+1], lw=0.5)
        }
      }
      else{ #origscale
        plot(ps:pe,rep(0,length(ps:pe)), col = 0, xlim = c(ps,pe),ylim = c(0,1))
        for (i in 1:ncol(rawdat)){
          lines(ps:pe, rawdat[ps:pe, i],col = c(4,2)[categ[i]+1], lw=0.5)
        }
        
      }
    }
  }
  
}

showBumpX = function(bumps, index=0,idx = NULL,  margin = 1,expand = 1.5,trend = TRUE, 
                    locs = NULL, chroms = NULL, samples = FALSE, rawdat = NULL, categ = NULL, origscale=FALSE,
                    thresh = NULL,usecex = 1,...) {
  if (index == 0  && is.null(idx)){
    error('Please specify either index or (start,end) of bump')
  }

  if (is.null(locs)){
    error('Please specify probe locations via "locs", or use showBump')
  }
  
  if (index==0){
    bs = as.numeric(idx)[1]
    be = as.numeric(idx)[2]
  } else {
    bs = bumps$table[index,"indexStart"]
    be = bumps$table[index,"indexEnd"]    
  }  
  
  # plots start and end based on margins

  # Expand range by 'expand/2' to each direction, and find 
  # probes in that region
  l_bs = locs[bs]
  l_be = locs[be]

  l_len = l_be - l_bs + 1
  l_ps = l_bs - ceiling(l_len*(expand/2))
  l_pe = l_be  + ceiling(l_len*(expand/2))

  probes_in_range = which(locs <= l_pe  & locs >= l_ps & chroms == chroms[bs])

  ps = min(probes_in_range)
  pe = max(probes_in_range)
  if (margin > 0) {
    ps_b = max(1,ps-margin)
    pe_b = min(pe+margin,length(bumps$coef))
    
    ps = min(ps,ps_b)
    pe = max(pe, pe_b)
  }

  
  cat(ps,pe)
  
  if (origscale){
    par(mfcol = c(2,1),mar = c(1,4,0,1), oma = c(3,0,1,0), bty = 'n',cex=usecex)
    
  }
  
  norm = 1000
  xpoints = locs[ps:pe]/norm
  if (samples) {
    plot(xpoints,rep(0,length(ps:pe)), xlim = c(l_ps, l_pe)/norm, col = 0,ylim = c(0,1),
         ylab = "% Meth",xaxt="n")
    for (i in 1:ncol(rawdat)){
      points(xpoints, rawdat[ps:pe, i],pch = c(1,2)[categ[i]+1], cex = 0.7,col = c(4,2)[categ[i]+1], lw=0.3)
      lines(xpoints, rawdat[ps:pe, i],col = c(4,2)[categ[i]+1], lw=0.3)
    }
    lines(xpoints, rowMeans(rawdat[ps:pe, categ==0]),col = 4, lw=2)
    lines(xpoints, rowMeans(rawdat[ps:pe, categ==1]),col = 2, lw=2)
    abline(v = c(l_bs, l_be)/norm, lt =2)
    abline(h = c(0,1), col = grey(0.5))
  }
  if (trend) {
    par(bty = 'n')
    plot(xpoints,bumps$coef[ps:pe],xlim = c(l_ps, l_pe)/norm,
         ylab = 'Mean Diff', ...)
    lines(xpoints,bumps$fitted[ps:pe],col=grey(0.7),lw=2)
    lines(locs[bs:be]/norm,bumps$fitted[bs:be],col=1,lw=3)
    abline(v = c(l_bs, l_be)/norm, lt =2)
    if (!is.null(thresh)){
      abline(h = c(-thresh,thresh), lt = 2, lw = 0.5)
      abline(h = c(0), lw = 2)
      
    }
  }
  mtext('Genomic Location (kb)',1,outer = TRUE,line = 1,cex = usecex )
  
  return(list(index_enlarge = ps:pe,index_bump = bs:be,locs_bump = locs[bs:be]))
}

showBumpInt = function(bumps, index=0,idx = NULL,  margin = 1,expand = 1.5,trend = TRUE, 
                     locs = NULL, chroms = NULL, samples = FALSE, rawdat = NULL, categ = NULL, origscale=FALSE,
                     thresh = NULL,usecex = 1,CI = NULL,
                     ...) {
  if (index == 0  && is.null(idx)){
    error('Please specify either index or (start,end) of bump')
  }
  
  if (is.null(locs)){
    error('Please specify probe locations via "locs", or use showBump')
  }
  
  if (index==0){
    bs = as.numeric(idx)[1]
    be = as.numeric(idx)[2]
  } else {
    bs = bumps$table[index,"indexStart"]
    be = bumps$table[index,"indexEnd"]    
  }  
  
  # plots start and end based on margins
  
  # Expand range by 'expand/2' to each direction, and find 
  # probes in that region
  l_bs = locs[bs]
  l_be = locs[be]
  
  l_len = l_be - l_bs + 1
  l_ps = l_bs - ceiling(l_len*(expand/2))
  l_pe = l_be  + ceiling(l_len*(expand/2))
  
  probes_in_range = which(locs <= l_pe  & locs >= l_ps & chroms == chroms[bs])
  
  ps = min(probes_in_range)
  pe = max(probes_in_range)
  if (margin > 0) {
    ps_b = max(1,ps-margin)
    pe_b = min(pe+margin,length(bumps$coef))
    
    ps = min(ps,ps_b)
    pe = max(pe, pe_b)
  }
  
  
  cat(ps,pe)
  
  if (origscale){
    par(mfcol = c(2,1),mar = c(1,4,0,1), oma = c(3,0,1,0), bty = 'n',cex=usecex)
    
  }
  
  norm = 1000
  xpoints = locs[ps:pe]/norm
  if (trend) {
    par(bty = 'n')
    plot(xpoints,bumps$coef[ps:pe],xlim = c(l_ps, l_pe)/norm,
         ylab = 'Difference', xaxt="n" ,...)
    lines(xpoints,bumps$fitted[ps:pe],col=grey(0.7),lw=3)
    lines(locs[bs:be]/norm,bumps$fitted[bs:be],col=1,lw=4)
    abline(v = c(l_bs, l_be)/norm, lt =2)
    if (!is.null(thresh)){
      abline(h = c(-thresh,thresh), lt = 2, lw = 2)
      abline(h = c(0), lw = 2)
      
    }
    if (!is.null(CI)){
      bump_range = range(locs[bs:be])/norm
      x_pos = mean(bump_range)
      x_min = x_pos - (bump_range[2]-bump_range[1])/2  *0.7
      x_max = x_pos + (bump_range[2]-bump_range[1])/2  *0.7
      
      lines(rep(x_pos,2),CI, col = 4)
      lines(c(x_min, x_max), c(CI[1],CI[1]), col = 4,lw = 2)
      lines(c(x_min, x_max), c(CI[2],CI[2]), col = 4,lw = 2)
    }
  }
  if (samples) {
    par(bty = "l")
    plot(xpoints,rep(0,length(ps:pe)), xlim = c(l_ps, l_pe)/norm, col = 0,ylim = c(0,1),
         ylab = "Methylation")
    for (i in 1:ncol(rawdat)){
      points(xpoints, rawdat[ps:pe, i],pch = c(1,2)[categ[i]+1], cex = 0.7,col = c(4,2)[categ[i]+1], lw=0.3)
      lines(xpoints, rawdat[ps:pe, i],col = c(4,2)[categ[i]+1], lw=0.3)
    }
    lines(xpoints, rowMeans(rawdat[ps:pe, categ==0]),col = 4, lw=2)
    lines(xpoints, rowMeans(rawdat[ps:pe, categ==1]),col = 2, lw=2)
    abline(v = c(l_bs, l_be)/norm, lt =2)
    abline(h = c(0,1), col = grey(0.5))
  }
  
  mtext('Genomic Location (kb)',1,outer = TRUE,line = 1,cex = usecex )
  
  return(list(index_enlarge = ps:pe,index_bump = bs:be,locs_bump = locs[bs:be]))
}

library(plotrix)
addCI = function(bottom, top, center = NULL, bump = bump,width = 0.9,
                 useplot = c(1,1), ...) {
  par(mfg = useplot)
  norm = 1000
  bump_range = range(bump$locs_bump)/norm
  x_pos = mean(bump_range)
  x_min = x_pos - (bump_range[2]-bump_range[1])/2  *width
  x_max = x_pos + (bump_range[2]-bump_range[1])/2  *width
  lines(rep(x_pos,2),c(bottom,top), col = 4)
  lines(c(x_min, x_max), c(bottom,bottom), col = 4,lw = 2)
  lines(c(x_min, x_max), c(top,top), col = 4,lw = 2)
}

